# Copyright (c) 2011 Leif Johnson <leif@leifjohnson.net>
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.


header = (block) ->
  parts = ["<thead><tr><td>"]
  for i in [0...16]
    parts.push "<th>#{i.toString 16}"
  parts.join ''


exports.search = (e) ->
  _unibrow.search $('#q').val()
  e.stopPropagation()


_fonts = {}

exports.loadFont = (e) ->
  font = $('#f').val()
  if font not of _fonts
    _fonts[font] = true
    li = document.createElement 'link'
    li.rel = 'stylesheet'
    li.type = 'text/css'
    li.async = true
    li.href = 'http://fonts.googleapis.com/css?family=' + font.replace /\\s+/g, '+'
    l = document.getElementsByTagName('link')[0]
    l.parentNode.insertBefore li, l
  $('#t').css 'font-family', font
  e.stopPropagation()


class Trie
  constructor: ->
    @payloads = []
    @children = {}

  add: (key, payload) ->
    @payloads.push payload
    if key.length > 0
      c = key[0]
      @children[c] = new Trie if not @children[c]
      @children[c].add key[1...key.length], payload

  find: (key) ->
    return @payloads if key.length is 0
    c = key[0]
    result = []
    if @children[c]
      result = @children[c].find key[1...key.length]
    return result


class exports.Browser
  constructor: (codepoints, query) ->
    @index = null
    $.ajax {
      url: codepoints
      dataType: 'json'
      error: (args...) =>
        console.log args
      success: (data) =>
        @rebuildIndex data, query
    }

  rebuildIndex: (annotations, query) =>
    @index = new Trie
    for cp, tokens of annotations
      for t in tokens.split ' '
        for i in [0...t.length]
          @index.add t[i...t.length], parseInt cp

    # clean out the top levels of the trie to prevent short searches.
    @index.payloads = []
    for c, node of @index.children
      node.payloads = []

    $('#q').val(query).change()

  search: (query) =>
    results = {}
    for t in query.toLowerCase().split ' '
      if t.match /^u?[0-9a-f]{1,4}$/
        if t[0] is 'u'
          t = t[1...t.length]
        if t.length is 1
          for i in [0...16]
            results[parseInt "#{t}#{i}00", 16] = true
        else
          while t.length < 4
            t += 0
          results[parseInt t, 16] = true
      else
        for cp in @index.find t
          results[cp] = true
    @display results

  display: (cps) ->
    blocks = {}
    for cp of cps
      cp = parseInt cp
      blocks[cp - (cp % 256)] = true
    $('#t').empty()
    for block of blocks
      b = block = parseInt block
      rows = []
      while b < block + 256
        name = (b & 0xfff0).toString 16
        while name.length < 4
          name = "0#{name}"
        row = ["<tr><th onclick=\"$('#q').val('#{b.toString 16}').change()\">#{name}</a>"]
        for i in [0...16]
          cls = if b of cps then 's' else ''
          row.push "<td class=\"#{cls}\">&#x#{b.toString 16};"
          b++
        rows.push row.join('')
      $('#t').append "<table>#{header block}<tbody>#{rows.join ''}"
