# Copyright (c) 2011 Leif Johnson <leif@leifjohnson.net>
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

express = require 'express'
stitch = require 'stitch'
stylus = require 'stylus'
uglify = require 'uglify-js'


package = stitch.createPackage
  paths: ["#{__dirname}/browser"]
  dependencies: ["#{__dirname}/public/jquery.js"]

package.createMinifiedServer = ->
  (req, res) =>
    @compile (err, source) ->
      if err
        console.error "#{err.stack}"
        message = "" + err.stack
        res.writeHead 500, 'Content-Type': 'text/javascript'
        res.end "throw #{JSON.stringify message}"
      else
        {gen_code, ast_squeeze, ast_mangle} = uglify.uglify
        res.writeHead 200, 'Content-Type': 'text/javascript'
        res.end gen_code ast_squeeze ast_mangle uglify.parser.parse source


exports.middleware = (options) ->
  app = express.createServer()

  app.configure ->
    app.set 'views', __dirname
    app.set 'view options', layout: false
    app.set 'view engine', 'jade'
    app.use express.logger 'short'
    app.use stylus.middleware(
      force: true
      debug: false
      compress: true
      src: "#{__dirname}/public"
      )
    app.use app.router
    app.use express.static("#{__dirname}/public")
    app.use express.errorHandler()

  app.get '/', (req, res) ->
    ctx = analytics: options?.analytics, query: req.param 'q', 'star'
    res.render 'main', ctx

  app.get '/unibrow.js', package.createMinifiedServer() # or .createServer()

  return app


if not module.parent
  app = exports.middleware()
  app.listen 3000
  console.log 'unibrow server listening on port 3000'
